(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.asyncro = global.asyncro || {})));
}(this, (function (exports) { 'use strict';

/** Invoke a list (object or array) of functions, returning their results in the same structure.
 *	@private
 */
function resolve(list) {
	var out = Array.isArray(list) ? [] : {};
	for (var i in list) {
		if (list.hasOwnProperty(i)) out[i] = list[i]();
	}return out;
}

/** reduce() callback that pushes values into an Array accumulator
 *	@private
 */
function pushReducer(acc, v) {
	return Promise.resolve().then(function () {
		return v;
	}).then(function (_resp) {
		acc.push(_resp);
		return acc;
	});
}

/** Invoke an async reducer function on each item in the given Array,
 *	where the reducer transforms an accumulator value based on each item iterated over.
 *	**Note:** because `reduce()` is order-sensitive, iteration is sequential.
 *
 *	> This is an asynchronous version of `Array.prototype.reduce()`
 *
 *	@param {Array} array			The Array to reduce
 *	@param {Function} reducer		Async function, gets passed `(accumulator, value, index, array)` and returns a new value for `accumulator`
 *	@param {Any} [accumulator]		Optional initial accumulator value
 *	@returns final `accumulator` value
 *
 *	@example
 *	await reduce(
 *		['/foo', '/bar', '/baz'],
 *		async (accumulator, value) => {
 *			accumulator[v] = await fetch(value);
 *			return accumulator;
 *		},
 *		{}
 *	);
 */
function reduce(array, reducer, accumulator) {
  function _recursive() {
    if (i < array.length) {
      return Promise.resolve().then(function () {
        return reducer(accumulator, array[i], i, array);
      }).then(function (_resp) {
        accumulator = _resp;
        i++;
        return _recursive();
      });
    }
  }

  var i;
  return Promise.resolve().then(function () {
    i = 0;
    return _recursive();
  }).then(function () {
    return accumulator;
  });
}

/** Invoke an async transform function on each item in the given Array **in parallel**,
 *	returning the resulting Array of mapped/transformed items.
 *
 *	> This is an asynchronous, parallelized version of `Array.prototype.map()`.
 *
 *	@param {Array} array			The Array to map over
 *	@param {Function} mapper		Async function, gets passed `(value, index, array)`, returns the new value.
 *	@returns {Array} resulting mapped/transformed values.
 *
 *	@example
 *	await map(
 *		['foo', 'baz'],
 *		async v => await fetch(v)
 *	)
 */
function map(array, mapper) {
  return Promise.all(array.map(mapper));
}

/** Invoke an async filter function on each item in the given Array **in parallel**,
 *	returning an Array of values for which the filter function returned a truthy value.
 *
 *	> This is an asynchronous, parallelized version of `Array.prototype.filter()`.
 *
 *	@param {Array} array			The Array to filter
 *	@param {Function} filterer		Async function. Gets passed `(value, index, array)`, returns true to keep the value in the resulting filtered Array.
 *	@returns {Array} resulting filtered values
 *
 *	@example
 *	await filter(
 *		['foo', 'baz'],
 *		async v => (await fetch(v)).ok
 *	)
 */
function filter(array, filterer) {
  var mapped;
  return Promise.resolve().then(function () {
    return map(array, filterer);
  }).then(function (_resp) {
    mapped = _resp;

    return array.filter(function (v, i) {
      return mapped[i];
    });
  });
}

/** Invoke all async functions in an Array or Object **in parallel**, returning the result.
 *	@param {Array<Function>|Object<Function>} list		Array/Object with values that are async functions to invoke.
 *	@returns {Array|Object} same structure as `list` input, but with values now resolved.
 *
 *	@example
 *	await parallel([
 *		async () => await fetch('foo'),
 *		async () => await fetch('baz')
 *	])
 */
function parallel(list) {
  return Promise.resolve().then(function () {
    return Promise.all(resolve(list));
  });
}

/** Invoke all async functions in an Array or Object **sequentially**, returning the result.
 *	@param {Array<Function>|Object<Function>} list		Array/Object with values that are async functions to invoke.
 *	@returns {Array|Object} same structure as `list` input, but with values now resolved.
 *
 *	@example
 *	await series([
 *		async () => await fetch('foo'),
 *		async () => await fetch('baz')
 *	])
 */
function series(list) {
  return Promise.resolve().then(function () {
    return reduce(resolve(list), pushReducer, []);
  });
}

exports.reduce = reduce;
exports.map = map;
exports.filter = filter;
exports.parallel = parallel;
exports.series = series;

})));
//# sourceMappingURL=asyncro.js.map
